"""
Sarmat.

Используемые константы.

Форматы значений.
"""
DATE_FORMAT = '%d.%m.%Y'
TIME_FORMAT = '%H:%M'
FULL_TIME_FORMAT = '%H:%M:%S'
DATETIME_FORMAT = f'{DATE_FORMAT} {TIME_FORMAT}'
FULL_DATETIME_FORMAT = f'{DATE_FORMAT} {FULL_TIME_FORMAT}'
